@@ -7,8 +7,7 @@ use ndarray::{
     aview1,
     arr2,
     Axis,
-    Ix,
-    Array,
+    Array2,
     ErrorKind,
 };
 
@@ -39,6 +38,6 @@ fn stacking() {
     let res = ndarray::stack(Axis(2), &[a.view(), c.view()]);
     assert_eq!(res.unwrap_err().kind(), ErrorKind::OutOfBounds);
 
-    let res: Result<Array<f64, (Ix, Ix)>, _> = ndarray::stack(Axis(0), &[]);
+    let res: Result<Array2<f64>, _> = ndarray::stack(Axis(0), &[]);
     assert_eq!(res.unwrap_err().kind(), ErrorKind::Unsupported);
 }
